@@ -19,19 +19,112 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
 #include "platform.h"
 
 #include <hugin_utils/utils.h>
+#include <vigra/imageinfo.hxx>
 
-#if defined __WXMAC__
+/** build filter string "*.ext", adds also upper case version for UNIX paths when needed */
+wxString GetFilterExtensions(const wxString& ext)
+{
+    wxString extensionString("*.");
+    extensionString.Append(ext);
+    if (wxFileName::IsCaseSensitive())
+    {
+        extensionString.Append(";*.").Append(ext.Upper());
+    };
+    return extensionString;
+};
+
+wxString GetVigraImageFilter()
+{
+    wxString filterString;
+    const std::string extensions = vigra::impexListExtensions();
+    std::vector<std::string> exts = hugin_utils::SplitString(extensions, " ");
+    if (exts.empty())
+    {
+        // if something goes wrong, add some defaults
+        exts.push_back("tif");
+        exts.push_back("tiff");
+        exts.push_back("jpg");
+        exts.push_back("jpeg");
+        exts.push_back("png");
+    }
+    filterString.Append(_("All Image files")).Append("|");
+    for (auto& ext : exts)
+    {
+        filterString.Append(GetFilterExtensions(ext)).Append(";");
+    };
+    // remove last character == ";"
+    filterString.RemoveLast();
+    filterString.Append("|");
+    return filterString;
+};
+
+wxString GetFileDialogImageFilters()
+{
+    wxString filterString=GetVigraImageFilter();
+    // append JPG/TIFF/PNG
+    filterString.Append(GetMainImageFilters());
+    filterString.Append("|").Append(_("HDR files (*.hdr)")).Append("|").Append(GetFilterExtensions("hdr"));
+    filterString.Append("|").Append(_("EXR files (*.exr)")).Append("|").Append(GetFilterExtensions("exr"));
+    filterString.Append("|").Append(_("All files (*)")).Append("|*");
+    return filterString;
+}
+
+wxString GetMainImageFilters()
+{
+    wxString filterString;
+    filterString.Append(_("JPEG files (*.jpg,*.jpeg)")).Append("|").Append(GetFilterExtensions("jpg")).Append(";").Append(GetFilterExtensions("jpeg"));
+    filterString.Append("|").Append(_("TIFF files (*.tif,*.tiff)")).Append("|").Append(GetFilterExtensions("tif")).Append(";").Append(GetFilterExtensions("tiff"));
+    filterString.Append("|").Append(_("PNG files (*.png)")).Append("|").Append(GetFilterExtensions("png"));
+    return filterString;
+}
+
+wxString GetFileDialogImageAndRawFilters()
+{
+    wxString filterString = GetVigraImageFilter();
+    // now the raw formats
+    const std::vector<std::string> exts = hugin_utils::GetRawExtensions();
+    filterString.Append(_("Raw files")).Append("|");
+    for (auto& ext : exts)
+    {
+        filterString.Append(GetFilterExtensions(ext)).Append(";");
+    };
+    // remove last character == ";"
+    filterString.RemoveLast();
+    filterString.Append("|");
+    // now add some image format at its own
+    filterString.Append(_("JPEG files (*.jpg,*.jpeg)")).Append("|").Append(GetFilterExtensions("jpg")).Append(";").Append(GetFilterExtensions("jpeg"));
+    filterString.Append("|").Append(_("TIFF files (*.tif,*.tiff)")).Append("|").Append(GetFilterExtensions("tif")).Append(";").Append(GetFilterExtensions("tiff"));
+    filterString.Append("|").Append(_("PNG files (*.png)")).Append("|").Append(GetFilterExtensions("png"));
+    filterString.Append("|").Append(_("HDR files (*.hdr)")).Append("|").Append(GetFilterExtensions("hdr"));
+    filterString.Append("|").Append(_("EXR files (*.exr)")).Append("|").Append(GetFilterExtensions("exr"));
+    return filterString;
+};
+
+bool IsRawExtension(const wxString& testExt)
+{
+    const std::vector<std::string> rawExts = hugin_utils::GetRawExtensions();
+    for (auto& ext : rawExts)
+    {
+        if (testExt.CmpNoCase(wxString(ext.c_str(), wxConvLocal)) == 0)
+        {
+            return true;
+        };
+    };
+    return false;
+};
+
+#if defined __WXMAC__ || defined __WXOSX_COCOA__
 
 #include <CoreFoundation/CFBundle.h>
-// next four only neccessary when "cout"ing to system log
+ #include "wx/osx/core/cfstring.h"
 #include <iostream>
 #include <stdio.h>
 #include "wx/utils.h"
@@ -49,7 +142,7 @@ CFStringRef MacCreateCFStringWithWxString(const wxString& string)
 
 wxString MacGetPathToMainExecutableFileOfBundle(CFStringRef bundlePath)
 {
-    wxString theResult = wxT("");
+    wxString theResult = wxEmptyString;
 
     CFURLRef bundleURL = CFURLCreateWithFileSystemPath(NULL, bundlePath, kCFURLPOSIXPathStyle, TRUE);
 
@@ -93,7 +186,7 @@ wxString MacGetPathToMainExecutableFileOfBundle(CFStringRef bundlePath)
                 else
                 {
                     CFRetain( pathInCFString );
-                    theResult =  wxMacCFStringHolder(pathInCFString).AsString(wxLocale::GetSystemEncoding());
+                    theResult =  wxCFStringRef(pathInCFString).AsString(wxLocale::GetSystemEncoding());
                     DEBUG_INFO("Mac: the executable's full path in the application bundle: " << theResult.mb_str(wxConvLocal));
                 }
             }
@@ -104,7 +197,7 @@ wxString MacGetPathToMainExecutableFileOfBundle(CFStringRef bundlePath)
 
 wxString MacGetPathToMainExecutableFileOfRegisteredBundle(CFStringRef BundleIdentifier)
 {
-    wxString theResult = wxT("");
+    wxString theResult = wxEmptyString;
 	
 	FSRef appRef;
 	CFURLRef bundleURL;
@@ -122,11 +215,11 @@ wxString MacGetPathToMainExecutableFileOfRegisteredBundle(CFStringRef BundleIden
 	if (err != noErr) {
 		// error, can't find PTBatcherGUI
 		cout << "PTBatcherGui check failed \n" << endl;
-		wxMessageBox(wxString::Format(_("External program %s not found in the bundle, reverting to system path"), wxT("open")), _("Error"));
+		hugin_utils::HuginMessageBox(wxString::Format(_("External program %s not found in the bundle, reverting to system path"), "open"), _("Hugin"), wxOK, wxGetActiveWindow());
 	}
     if(bundleURL == NULL)
     {
-        DEBUG_INFO("Mac: CFURL from string (" << bundlePath << ") failed." );
+        DEBUG_INFO("Mac: CFURL from string (" << bundleURL << ") failed." );
         return theResult;
     }
     
@@ -135,7 +228,7 @@ wxString MacGetPathToMainExecutableFileOfRegisteredBundle(CFStringRef BundleIden
     
     if(bundle == NULL)
     {
-        DEBUG_INFO("Mac: CFBundleCreate (" << bundlePath << " ) failed" );
+        DEBUG_INFO("Mac: CFBundleCreate (" << bundleURL << " ) failed" );
     }
     else
     {
@@ -164,7 +257,7 @@ wxString MacGetPathToMainExecutableFileOfRegisteredBundle(CFStringRef BundleIden
                 else
                 {
                     CFRetain( pathInCFString );
-                    theResult =  wxMacCFStringHolder(pathInCFString).AsString(wxLocale::GetSystemEncoding());
+                    theResult =  wxCFStringRef(pathInCFString).AsString(wxLocale::GetSystemEncoding());
                     DEBUG_INFO("Mac: the executable's full path in the application bundle: " << theResult.mb_str(wxConvLocal));
                 }
             }
@@ -178,7 +271,7 @@ wxString MacGetPathToMainExecutableFileOfRegisteredBundle(CFStringRef BundleIden
 
 wxString MacGetPathToBundledAppMainExecutableFile(CFStringRef appname)
 {
-    wxString theResult = wxT("");
+    wxString theResult = wxEmptyString;
 
     CFBundleRef mainbundle = CFBundleGetMainBundle();
     if(mainbundle == NULL)
@@ -229,7 +322,7 @@ wxString MacGetPathToBundledAppMainExecutableFile(CFStringRef appname)
                         else
                         {
                             CFRetain( pathInCFString );
-                            theResult =  wxMacCFStringHolder(pathInCFString).AsString(wxLocale::GetSystemEncoding());
+                            theResult =  wxCFStringRef(pathInCFString).AsString(wxLocale::GetSystemEncoding());
                             DEBUG_INFO("Mac: the executable's full path in the application bundle: " << theResult.mb_str(wxConvLocal));
                         }
                     }
@@ -242,7 +335,7 @@ wxString MacGetPathToBundledAppMainExecutableFile(CFStringRef appname)
 
 wxString MacGetPathToBundledResourceFile(CFStringRef filename)
 {
-    wxString theResult = wxT("");
+    wxString theResult = wxEmptyString;
 
     CFBundleRef mainbundle = CFBundleGetMainBundle();
     if(mainbundle == NULL)
@@ -267,7 +360,7 @@ wxString MacGetPathToBundledResourceFile(CFStringRef filename)
             else
             {
                 CFRetain( pathInCFString );
-                theResult = wxMacCFStringHolder(pathInCFString).AsString(wxLocale::GetSystemEncoding());
+                theResult = wxCFStringRef(pathInCFString).AsString(wxLocale::GetSystemEncoding());
                 DEBUG_INFO("Mac: the resource file's path in the application bundle: " << theResult.mb_str(wxConvLocal));
             }
         }
@@ -275,9 +368,44 @@ wxString MacGetPathToBundledResourceFile(CFStringRef filename)
     return theResult;
 }
 
+wxString MacGetPathToBundledFrameworksDirectory()
+{
+    wxString theResult = wxEmptyString;
+    
+    CFBundleRef mainbundle = CFBundleGetMainBundle();
+    if(mainbundle == NULL)
+    {
+        DEBUG_INFO("Mac: Not bundled");
+    }
+    else
+    {
+        CFURLRef XRCurl = CFBundleCopyBundleURL(mainbundle);
+        if(XRCurl == NULL)
+        {
+            DEBUG_INFO("Mac: Cannot locate the file in bundle.");
+        }
+        else
+        {
+            CFStringRef pathInCFString = CFURLCopyFileSystemPath(XRCurl, kCFURLPOSIXPathStyle);
+            CFRelease( XRCurl );
+            if(pathInCFString == NULL)
+            {
+                DEBUG_INFO("Mac: Failed to get URL in CFString");
+            }
+            else
+            {
+                CFRetain( pathInCFString );
+                theResult = wxCFStringRef(pathInCFString).AsString(wxLocale::GetSystemEncoding());
+                DEBUG_INFO("Mac: the Frameworks file's path in the application bundle: " << theResult.mb_str(wxConvLocal));
+            }
+        }
+    }
+    return theResult + "/Contents/Frameworks";
+}
+
 wxString MacGetPathToBundledExecutableFile(CFStringRef filename)
 {
-    wxString theResult = wxT("");
+    wxString theResult = wxEmptyString;
 
     CFBundleRef mainbundle = CFBundleGetMainBundle();
     if(mainbundle == NULL)
@@ -309,7 +437,7 @@ wxString MacGetPathToBundledExecutableFile(CFStringRef filename)
                 else
                 {
                     CFRetain( pathInCFString );
-                    theResult =  wxMacCFStringHolder(pathInCFString).AsString(wxLocale::GetSystemEncoding());
+                    theResult =  wxCFStringRef(pathInCFString).AsString(wxLocale::GetSystemEncoding());
                     DEBUG_INFO("Mac: executable's full path in the application bundle: " << theResult.mb_str(wxConvLocal));
                 }
             }
@@ -321,7 +449,7 @@ wxString MacGetPathToBundledExecutableFile(CFStringRef filename)
 
 wxString MacGetPathToUserDomainTempDir()
 {
-    wxString tmpDirPath = wxT("");
+    wxString tmpDirPath = wxEmptyString;
     
     FSRef tempDirRef;
     OSErr err = FSFindFolder(kUserDomain, kTemporaryFolderType, kCreateFolder, &tempDirRef);
@@ -332,8 +460,7 @@ wxString MacGetPathToUserDomainTempDir()
         {
             CFStringRef tmpPath = CFURLCopyFileSystemPath(tempDirURL, kCFURLPOSIXPathStyle);
             CFRetain(tmpPath);
-            tmpDirPath = wxMacCFStringHolder(tmpPath).AsString(wxLocale::GetSystemEncoding());
-            
+            tmpDirPath = wxCFStringRef(tmpPath).AsString(wxLocale::GetSystemEncoding());
             CFRelease(tempDirURL);
         }
     }
@@ -343,7 +470,7 @@ wxString MacGetPathToUserDomainTempDir()
 
 wxString MacGetPathToUserAppSupportAutoPanoFolder()
 {
-    wxString appSupportAutoPanoFolder = wxT("");
+    wxString appSupportAutoPanoFolder = wxEmptyString;
 
     FSRef appSupportFolder;
     OSErr err = FSFindFolder(kUserDomain,kApplicationSupportFolderType,kDontCreateFolder,&appSupportFolder);
@@ -354,8 +481,7 @@ wxString MacGetPathToUserAppSupportAutoPanoFolder()
         CFURLRef autopanoURL = CFURLCreateCopyAppendingPathComponent(kCFAllocatorDefault,appSupportHugin,CFSTR("Autopano"),true);
         CFStringRef tmpPath = CFURLCopyFileSystemPath(autopanoURL,  kCFURLPOSIXPathStyle);
         CFRetain(tmpPath);
-        appSupportAutoPanoFolder = wxMacCFStringHolder(tmpPath).AsString(wxLocale::GetSystemEncoding());
-
+        appSupportAutoPanoFolder = wxCFStringRef(tmpPath).AsString(wxLocale::GetSystemEncoding());
         CFRelease(autopanoURL);
     }
     return appSupportAutoPanoFolder;
@@ -370,14 +496,14 @@ const wxString getInvalidCharacters()
 {
 #if defined __WXMSW__
     // the characters :"*?<>| are not allowed in filenames, these are handled well by the file dialog
-    // we need only to check for characters, which does not work with the makefiles
-    return wxT("=;%");
+    // all other characters should work
+    return wxEmptyString;
 #else
     // the characters =;:% does not work with the makefile
     // we are also rejecting the characters <>*?| which are principally allowed in filenames but will probably make problems when used
     // the double quote does not work with the panotools file format, so also reject
     //@BUG tilde ~ and backslash \ are not working with vigraimpex, if this works again these characters can be removed from the list
-    return wxT("=;:%*?<>|\"\\~");
+    return "*?<>|\"\\~";
 #endif
 };
 
@@ -393,3 +519,31 @@ bool containsInvalidCharacters(const wxString stringToTest)
     };
     return false;
 };
+
+void ShowFilenameWarning(wxWindow* parent, const wxArrayString filelist)
+{
+    wxDialog dlg;
+    wxXmlResource::Get()->LoadDialog(&dlg, parent, "dlg_warning_filename");
+    XRCCTRL(dlg, "dlg_warning_text", wxStaticText)->SetLabel(wxString::Format(_("The filename(s) contains one of the following invalid characters: %s\nHugin can not work with these filenames. Please rename your file(s) and try again."), getInvalidCharacters().c_str()));
+    XRCCTRL(dlg, "dlg_warning_list", wxListBox)->Append(filelist);
+    dlg.Fit();
+    dlg.CenterOnScreen();
+    dlg.ShowModal();
+};
+
+#if wxUSE_ON_FATAL_EXCEPTION
+void GenerateReport(wxDebugReport::Context ctx)
+{
+    //from the debugrpt sample inspired
+    wxDebugReportCompress report;
+    report.AddAll(ctx);
+    if (wxDebugReportPreviewStd().Show(report))
+    {
+        if (report.Process())
+        {
+            wxLogMessage(_("Debug report generated in \"%s\"."), report.GetCompressedFileName().c_str());
+            report.Reset();
+        };
+    };
+};
+#endif
