@@ -20,8 +20,8 @@
  *  Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -41,26 +41,11 @@ class AboutDialog: public wxDialog
 {
 public:
     /** Constructor, read from xrc ressource */
-    AboutDialog(wxWindow *parent);
+    explicit AboutDialog(wxWindow *parent);
 
 private:
-
-    int m_mode;
-    wxString m_logo_file;
-    wxNotebook* m_about_notebook;
-    wxStaticBitmap * m_logoImgCtrl;
-    wxBitmap m_logo;
-
-    DECLARE_EVENT_TABLE()
-
-    /** retrieves the system informations */
+    /** retrieves the system information */
     void GetSystemInformation(wxFont *font);
-    /** event handler for changing tab */
-    void OnChangedTab(wxNotebookEvent &e);
-    /** function to switch the logo image based on selected tab */
-    void SetMode(int newMode);
-    /** function to set the log image */
-    void SetLogo(wxString newLogoFile);
 };
 
 #endif // _ABOUTDIALOG_H
