@@ -25,7 +25,9 @@
  */
 
 #include "hugin/ImageVariableDialog.h"
+#include "base_wx/GraphTools.h"
 #include "base_wx/wxPlatform.h"
+#include "base_wx/wxutils.h"
 #include "panoinc.h"
 #include "photometric/ResponseTransform.h"
 #include <map>
@@ -34,32 +36,17 @@
 #include "hugin/huginApp.h"
 #include "base_wx/CommandHistory.h"
 #include "base_wx/PanoCommand.h"
-#include "panotools/PanoToolsInterface.h"
-
-BEGIN_EVENT_TABLE(ImageVariableDialog,wxDialog)
-    EVT_BUTTON(wxID_OK, ImageVariableDialog::OnOk)
-    EVT_BUTTON(wxID_HELP, ImageVariableDialog::OnHelp)
-    EVT_BUTTON(XRCID("image_show_distortion_graph"), ImageVariableDialog::OnShowDistortionGraph)
-    EVT_BUTTON(XRCID("image_show_vignetting_graph"), ImageVariableDialog::OnShowVignettingGraph)
-    EVT_BUTTON(XRCID("image_show_response_graph"), ImageVariableDialog::OnShowResponseGraph)
-END_EVENT_TABLE()
 
 ImageVariableDialog::ImageVariableDialog(wxWindow *parent, HuginBase::Panorama* pano, HuginBase::UIntSet imgs)
 {
     // load our children. some children might need special
     // initialization. this will be done later.
-    wxXmlResource::Get()->LoadDialog(this, parent, wxT("image_variables_dialog"));
+    wxXmlResource::Get()->LoadDialog(this, parent, "image_variables_dialog");
 
-#ifdef __WXMSW__
-    wxIcon myIcon(huginApp::Get()->GetXRCPath() + wxT("data/hugin.ico"),wxBITMAP_TYPE_ICO);
-#else
-    wxIcon myIcon(huginApp::Get()->GetXRCPath() + wxT("data/hugin.png"),wxBITMAP_TYPE_PNG);
-#endif
-    SetIcon(myIcon);
     wxConfigBase * cfg = wxConfigBase::Get();
     //position
-    int x = cfg->Read(wxT("/ImageVariablesDialog/positionX"),-1l);
-    int y = cfg->Read(wxT("/ImageVariablesDialog/positionY"),-1l);
+    int x = cfg->Read("/ImageVariablesDialog/positionX",-1l);
+    int y = cfg->Read("/ImageVariablesDialog/positionY",-1l);
     if ( y >= 0 && x >= 0) 
     {
         this->Move(x, y);
@@ -72,14 +59,21 @@ ImageVariableDialog::ImageVariableDialog(wxWindow *parent, HuginBase::Panorama*
     m_images=imgs;
     m_popup=NULL;
     InitValues();
+    // bind event handler
+    Bind(wxEVT_BUTTON, &ImageVariableDialog::OnOk, this, wxID_OK);
+    Bind(wxEVT_BUTTON, &ImageVariableDialog::OnHelp, this, wxID_HELP);
+    Bind(wxEVT_BUTTON, &ImageVariableDialog::OnShowDistortionGraph, this, XRCID("image_show_distortion_graph"));
+    Bind(wxEVT_BUTTON, &ImageVariableDialog::OnShowVignettingGraph, this, XRCID("image_show_vignetting_graph"));
+    Bind(wxEVT_BUTTON, &ImageVariableDialog::OnShowResponseGraph, this, XRCID("image_show_response_graph"));
+    Bind(wxEVT_CHOICE, &ImageVariableDialog::OnResponseSelected, this, XRCID("image_variable_responseType"));
 };
 
 ImageVariableDialog::~ImageVariableDialog()
 {
     wxConfigBase * cfg = wxConfigBase::Get();
     wxPoint ps = this->GetPosition();
-    cfg->Write(wxT("/ImageVariablesDialog/positionX"), ps.x);
-    cfg->Write(wxT("/ImageVariablesDialog/positionY"), ps.y);
+    cfg->Write("/ImageVariablesDialog/positionX", ps.x);
+    cfg->Write("/ImageVariablesDialog/positionY", ps.y);
     cfg->Flush();
 };
 
@@ -93,7 +87,7 @@ wxTextCtrl* GetImageVariableControl(const wxWindow* parent, const char* varname)
     return wxStaticCast(
                 parent->FindWindow(
                     wxXmlResource::GetXRCID(
-                        wxString(wxT("image_variable_")).append(wxString(varname, wxConvLocal)).c_str()
+                        wxString("image_variable_").append(wxString(varname, wxConvLocal)).c_str()
                     )
                 ), wxTextCtrl
            );
@@ -101,7 +95,7 @@ wxTextCtrl* GetImageVariableControl(const wxWindow* parent, const char* varname)
 
 void ImageVariableDialog::InitValues()
 {
-    if(m_images.size()==0)
+    if(m_images.empty())
     {
         return;
     };
@@ -117,9 +111,9 @@ void ImageVariableDialog::InitValues()
         XRCCTRL(*this, "image_variable_responseType", wxChoice)->SetSelection(responseType);
     };
 
-    int degDigits = wxConfigBase::Get()->Read(wxT("/General/DegreeFractionalDigitsEdit"),3);
-    int pixelDigits = wxConfigBase::Get()->Read(wxT("/General/PixelFractionalDigitsEdit"),2);
-    int distDigitsEdit = wxConfigBase::Get()->Read(wxT("/General/DistortionFractionalDigitsEdit"),5);
+    int degDigits = wxConfigBase::Get()->Read("/General/DegreeFractionalDigitsEdit",3);
+    int pixelDigits = wxConfigBase::Get()->Read("/General/PixelFractionalDigitsEdit",2);
+    int distDigitsEdit = wxConfigBase::Get()->Read("/General/DistortionFractionalDigitsEdit",5);
     
     HuginBase::VariableMapVector imgVarVector = m_pano->getVariables();
 
@@ -149,6 +143,8 @@ void ImageVariableDialog::InitValues()
             GetImageVariableControl(this, *varname)->SetValue(hugin_utils::doubleTowxString(val,ndigits));
         };
     };
+    wxCommandEvent dummy;
+    OnResponseSelected(dummy);
 };
 
 void ImageVariableDialog::SetGuiLevel(GuiLevel newLevel)
@@ -187,6 +183,11 @@ bool ImageVariableDialog::ApplyNewVariables()
             {
                 if(strcmp(*varname, "v")==0)
                 {
+                    if (val < 0.05)
+                    {
+                        hugin_utils::HuginMessageBox(_("The horizontal field of view must be positive."), _("Hugin"), wxICON_EXCLAMATION | wxOK, this);
+                        return false;
+                    };
                     switch(m_pano->getImage(*m_images.begin()).getProjection())
                     {
                         case HuginBase::SrcPanoImage::RECTILINEAR:
@@ -198,14 +199,9 @@ bool ImageVariableDialog::ApplyNewVariables()
                         case HuginBase::SrcPanoImage::FISHEYE_ORTHOGRAPHIC:
                             if(val>190)
                             {
-                                if(wxMessageBox(
+                                if (hugin_utils::HuginMessageBox(
                                     wxString::Format(_("You have given a field of view of %.2f degrees.\n But the orthographic projection is limited to a field of view of 180 degress.\nDo you want still use that high value?"), val),
-#ifdef __WXMSW__
-                                    _("Hugin"),
-#else
-                                    wxT(""),
-#endif
-                                    wxICON_EXCLAMATION | wxYES_NO)==wxNO)
+                                    _("Hugin"), wxICON_EXCLAMATION | wxYES_NO, this) == wxNO)
                                 {
                                     return false;
                                 };
@@ -236,7 +232,7 @@ bool ImageVariableDialog::ApplyNewVariables()
         }
         commands.push_back(new PanoCommand::UpdateSrcImagesCmd( *m_pano, m_images, SrcImgs ));
     }
-    if(varMap.size()>0)
+    if(!varMap.empty())
     {
         for(HuginBase::UIntSet::const_iterator it=m_images.begin();it!=m_images.end();++it)
         {
@@ -245,7 +241,7 @@ bool ImageVariableDialog::ApplyNewVariables()
                 );
         };
     };
-    if(commands.size()>0)
+    if(!commands.empty())
     {
         PanoCommand::GlobalCmdHist::getInstance().addCommand(new PanoCommand::CombinedPanoCommand(*m_pano, commands));
         return true;
@@ -271,209 +267,46 @@ void ImageVariableDialog::OnHelp(wxCommandEvent & e)
     {
         //lens parameters
         case 1:
-            MainFrame::Get()->DisplayHelp(wxT("Lens_correction_model.html"));
+            MainFrame::Get()->DisplayHelp("Lens_correction_model.html");
             break;
         case 2:
-            MainFrame::Get()->DisplayHelp(wxT("Vignetting.html"));
+            MainFrame::Get()->DisplayHelp("Vignetting.html");
             break;
         case 3:
-            MainFrame::Get()->DisplayHelp(wxT("Camera_response_curve.html"));
+            MainFrame::Get()->DisplayHelp("Camera_response_curve.html");
             break;
         default:
-            MainFrame::Get()->DisplayHelp(wxT("Image_positioning_model.html"));
+            MainFrame::Get()->DisplayHelp("Image_positioning_model.html");
             break;
     };
 };
 
+void ImageVariableDialog::OnResponseSelected(wxCommandEvent& e)
+{
+    const bool showResponseParam = XRCCTRL(*this, "image_variable_responseType", wxChoice)->GetSelection() == 0;
+    wxTextCtrl* control = XRCCTRL(*this, "image_variable_Ra", wxTextCtrl);
+    control->Enable(showResponseParam);
+    control->Show(showResponseParam);
+    control = XRCCTRL(*this, "image_variable_Rb", wxTextCtrl);
+    control->Enable(showResponseParam);
+    control->Show(showResponseParam);
+    control = XRCCTRL(*this, "image_variable_Rc", wxTextCtrl);
+    control->Enable(showResponseParam);
+    control->Show(showResponseParam);
+    control = XRCCTRL(*this, "image_variable_Rd", wxTextCtrl);
+    control->Enable(showResponseParam);
+    control->Show(showResponseParam);
+    control = XRCCTRL(*this, "image_variable_Re", wxTextCtrl);
+    control->Enable(showResponseParam);
+    control->Show(showResponseParam);
+};
+
 const char *ImageVariableDialog::m_varNames[] = { "y", "p", "r", "TrX", "TrY", "TrZ", "Tpy", "Tpp", 
                                   "v", "a", "b", "c", "d", "e", "g", "t",
                                   "Eev", "Er", "Eb", 
                                   "Vb", "Vc", "Vd", "Vx", "Vy",
                                   "Ra", "Rb", "Rc", "Rd", "Re", 0};
 
-IMPLEMENT_CLASS(GraphPopupWindow, wxPopupTransientWindow)
-
-BEGIN_EVENT_TABLE(GraphPopupWindow, wxPopupTransientWindow)
-EVT_LEFT_DOWN(GraphPopupWindow::OnLeftDown)
-END_EVENT_TABLE()
-
-GraphPopupWindow::GraphPopupWindow(wxWindow* parent, wxBitmap bitmap) : wxPopupTransientWindow(parent)
-{
-    wxPanel* panel=new wxPanel(this);
-    m_bitmapControl=new wxStaticBitmap(panel, wxID_ANY, bitmap);
-    panel->Connect(wxEVT_LEFT_DOWN, wxMouseEventHandler(GraphPopupWindow::OnLeftDown), NULL, this);
-    m_bitmapControl->Connect(wxEVT_LEFT_DOWN, wxMouseEventHandler(GraphPopupWindow::OnLeftDown), NULL, this);
-    m_bitmapControl->Connect(wxEVT_RIGHT_DOWN, wxMouseEventHandler(GraphPopupWindow::OnRightDown), NULL, this);
-    wxBoxSizer* topsizer=new wxBoxSizer(wxHORIZONTAL);
-    topsizer->Add(m_bitmapControl);
-    panel->SetSizer(topsizer);
-    topsizer->Fit(panel);
-    topsizer->Fit(this);
-};
-
-void GraphPopupWindow::OnLeftDown(wxMouseEvent &e)
-{
-    Dismiss();
-};
-
-void GraphPopupWindow::OnRightDown(wxMouseEvent &e)
-{
-    wxConfigBase* config=wxConfigBase::Get();
-    wxFileDialog dlg(this,
-                        _("Save graph"),
-                        config->Read(wxT("/actualPath"),wxT("")), wxT(""),
-                        _("Bitmap (*.bmp)|*.bmp|PNG-File (*.png)|*.png"),
-                        wxFD_SAVE | wxFD_OVERWRITE_PROMPT, wxDefaultPosition);
-    dlg.SetDirectory(config->Read(wxT("/actualPath"),wxT("")));
-    dlg.SetFilterIndex(config->Read(wxT("/lastImageTypeIndex"), 0l));
-    if (dlg.ShowModal() == wxID_OK)
-    {
-        config->Write(wxT("/actualPath"), dlg.GetDirectory());  // remember for later
-        wxFileName filename(dlg.GetPath());
-        int imageType=dlg.GetFilterIndex();
-        config->Write(wxT("/lastImageTypeIndex"), imageType);
-        if(!filename.HasExt())
-        {
-            switch(imageType)
-            {
-                case 1:
-                    filename.SetExt(wxT("png"));
-                    break;
-                case 0:
-                default:
-                    filename.SetExt(wxT("bmp"));
-                    break;
-            };
-        };
-        if (filename.FileExists())
-        {
-            int d = wxMessageBox(wxString::Format(_("File %s exists. Overwrite?"), filename.GetFullPath().c_str()),
-                                 _("Save image"), wxYES_NO | wxICON_QUESTION);
-            if (d != wxYES)
-            {
-                return;
-            }
-        }
-        switch(imageType)
-        {
-            case 1:
-                m_bitmapControl->GetBitmap().SaveFile(filename.GetFullPath(), wxBITMAP_TYPE_PNG);
-                break;
-            case 0:
-            default:
-                m_bitmapControl->GetBitmap().SaveFile(filename.GetFullPath(), wxBITMAP_TYPE_BMP);
-                break;
-        };
-    };
-};
-
-/**help class to draw charts */
-class Graph
-{
-public:
-    /** constructors, set size and background colour of resulting bitmap */
-    Graph(int graphWidth, int graphHeight, wxColour backgroundColour)
-    {
-        m_width=graphWidth;
-        m_height=graphHeight;
-        //create bitmap
-        m_bitmap=new wxBitmap(m_width, m_height);
-        m_dc.SelectObject(*m_bitmap);
-        m_dc.SetBackground(wxBrush(backgroundColour));
-        m_dc.Clear();
-        //draw border
-        m_dc.SetPen(wxPen(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOWTEXT)));
-        m_dc.SetBrush(*wxTRANSPARENT_BRUSH);
-        m_dc.DrawRectangle(0, 0, m_width, m_height);
-        SetChartArea(0,0, m_width, m_height);
-        SetChartDisplay(0,0,1,1);
-    };
-
-    /** destructor */
-    ~Graph()
-    {
-        m_dc.SelectObject(wxNullBitmap);
-    };
-
-    /** set where to draw the chart on the bitmap */
-    void SetChartArea(int left, int top, int right, int bottom)
-    {
-        m_dc.DestroyClippingRegion();
-        m_left=left;
-        m_top=top;
-        m_right=right;
-        m_bottom=bottom;
-        m_dc.SetClippingRegion(m_left-1, m_top-1, m_right-m_left+2, m_bottom-m_top+2);
-    };
-
-    /** set the real dimension of the chart */
-    void SetChartDisplay(double xmin, double ymin, double xmax, double ymax)
-    {
-        m_xmin=xmin;
-        m_ymin=ymin;
-        m_xmax=xmax;
-        m_ymax=ymax;
-    };
-
-    /** draws the grid with linesX lines in x-direction and linexY lines in y-direction */
-    void DrawGrid(size_t linesX, size_t linesY)
-    {
-        m_dc.SetPen(wxPen(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOWTEXT)));
-        for(size_t i=0; i<linesX; i++)
-        {
-            int x=TransformX((double)i*(m_xmax-m_xmin)/(linesX-1)+m_xmin);
-            m_dc.DrawLine(x, m_top, x, m_bottom);
-        };
-        for(size_t i=0; i<linesY; i++)
-        {
-            int y=TransformY((double)i*(m_ymax-m_ymin)/(linesY-1)+m_ymin);
-            m_dc.DrawLine(m_left, y, m_right, y);
-        };
-    };
-
-    /** draws a line with the coordinates given in points */
-    void DrawLine(std::vector<hugin_utils::FDiff2D> points, wxColour colour, int penWidth=1)
-    {
-        if(points.size()<2)
-        {
-            return;
-        };
-        wxPoint *polygonPoints=new wxPoint[points.size()];
-        for(size_t i=0; i<points.size(); i++)
-        {
-            polygonPoints[i].x=TransformX(points[i].x);
-            polygonPoints[i].y=TransformY(points[i].y);
-        };
-        m_dc.SetPen(wxPen(colour, penWidth));
-        m_dc.DrawLines(points.size(), polygonPoints);
-        delete []polygonPoints;
-    };
-
-    const wxBitmap GetGraph() const { return *m_bitmap; };
-private:
-    // prevent copying of class
-    Graph(const Graph&);
-    Graph& operator=(const Graph&);
-    //helper function to transform coordinates from real world to bitmap
-    int TransformX(double x)
-    {
-        return hugin_utils::round((x-m_xmin)/(m_xmax-m_xmin)*(m_right-m_left)+m_left);
-    };
-    int TransformY(double y)
-    {
-        return hugin_utils::round(m_bottom - (y - m_ymin) / (m_ymax - m_ymin)*(m_bottom - m_top));
-    };
-    // area to be drawn
-    double m_xmin, m_xmax, m_ymin, m_ymax;
-    // size of canvas
-    int m_width, m_height;
-    // chart area
-    int m_left, m_top, m_right, m_bottom;
-    // bitmap
-    wxBitmap* m_bitmap;
-    wxMemoryDC m_dc;
-};
-
 void ImageVariableDialog::OnShowDistortionGraph(wxCommandEvent & e)
 {
     wxString stringa=GetImageVariableControl(this, "a")->GetValue();
@@ -494,56 +327,14 @@ void ImageVariableDialog::OnShowDistortionGraph(wxCommandEvent & e)
 
     //create transformation
     HuginBase::SrcPanoImage srcImage;
-#define NRPOINTS 100
     srcImage.setSize(m_pano->getImage(*(m_images.begin())).getSize());
     // set projection to rectilinear, just in case, it should be the default value
     srcImage.setProjection(HuginBase::SrcPanoImage::RECTILINEAR);
     srcImage.setRadialDistortion(radialDist);
-    HuginBase::PanoramaOptions opts;
-    opts.setHFOV(srcImage.getHFOV());
-    opts.setProjection(HuginBase::PanoramaOptions::RECTILINEAR);
-    opts.setWidth(srcImage.getWidth());
-    opts.setHeight(srcImage.getHeight());
-    HuginBase::PTools::Transform transform;
-    transform.createTransform(srcImage, opts);
 
-    const double minLength = std::min<double>(srcImage.getWidth(), srcImage.getHeight()) / 2.0;
-    const double maxR = std::sqrt(static_cast<double>(srcImage.getWidth()*srcImage.getWidth() + srcImage.getHeight()*srcImage.getHeight()) )/ (2.0*minLength);
-    //draw graph
     delete m_popup;
-    Graph graph(300, 200, wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW));
-    graph.SetChartArea(10, 10, 290, 190);
-    graph.SetChartDisplay(0, -0.1, maxR, 0.1);
-    graph.DrawGrid(6, 7);
-    std::vector<hugin_utils::FDiff2D> points;
-    // draw helper lines
-    // help line at r=1
-    points.push_back(hugin_utils::FDiff2D(1, -0.1));
-    points.push_back(hugin_utils::FDiff2D(1, 0.1));
-    graph.DrawLine(points, wxSystemSettings::GetColour(wxSYS_COLOUR_GRAYTEXT), 1);
-    points.clear();
-    // helper line at r=width/height (for landscape images), maximal radius in horizontal (or vertical) direction
-    // chart goes to r for diagonal
-    const double rMaxHorzVert = std::max<double>(srcImage.getWidth(), srcImage.getHeight()) / (2.0*minLength);
-    points.push_back(hugin_utils::FDiff2D(rMaxHorzVert, -0.1));
-    points.push_back(hugin_utils::FDiff2D(rMaxHorzVert, 0.1));
-    graph.DrawLine(points, wxSystemSettings::GetColour(wxSYS_COLOUR_GRAYTEXT), 1);
-    points.clear();
-    // now draw distortion graph
-    double r = 0;
-    for(size_t i=0; i<NRPOINTS; i++)
-    {
-        double x,y;
-        if (transform.transform(x, y, r*minLength, 0))
-        {
-            points.push_back(hugin_utils::FDiff2D(r, x/minLength - r));
-        };
-        r += maxR / (NRPOINTS-1);
-    };
-    graph.DrawLine(points, wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOWTEXT), 2);
-
     //show popup
-    m_popup = new GraphPopupWindow(this, graph.GetGraph());
+    m_popup = new wxGraphTools::GraphPopupWindow(this, wxGraphTools::GetDistortionGraph(srcImage));
     wxWindow *button = (wxWindow*) e.GetEventObject();
     wxPoint pos=button->ClientToScreen(wxPoint(0,0));
     m_popup->Position(pos, button->GetSize());
@@ -568,7 +359,7 @@ void ImageVariableDialog::OnShowVignettingGraph(wxCommandEvent & e)
         return;
     };
     delete m_popup;
-    Graph graph(300, 200, wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW));
+    wxGraphTools::Graph graph(300, 200, wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW));
     graph.SetChartArea(10, 10, 290, 190);
     graph.SetChartDisplay(0, 0.8, 1, 1);
     graph.DrawGrid(6, 6);
@@ -576,6 +367,7 @@ void ImageVariableDialog::OnShowVignettingGraph(wxCommandEvent & e)
     //create ResponseTransform with vignetting only
     HuginBase::SrcPanoImage srcImage;
     srcImage.setRadialVigCorrCoeff(vigCorr);
+#define NRPOINTS 100
     srcImage.setSize(vigra::Size2D(2*NRPOINTS, 2*NRPOINTS));
     HuginBase::Photometric::ResponseTransform<double> transform(srcImage);
     transform.enforceMonotonicity();
@@ -590,7 +382,7 @@ void ImageVariableDialog::OnShowVignettingGraph(wxCommandEvent & e)
     graph.DrawLine(points, wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOWTEXT), 2);
 
     //display popup
-    m_popup = new GraphPopupWindow(this, graph.GetGraph());
+    m_popup = new wxGraphTools::GraphPopupWindow(this, graph.GetGraph());
     wxWindow *button = (wxWindow*) e.GetEventObject();
     wxPoint pos=button->ClientToScreen(wxPoint(0,0));
     m_popup->Position(pos, button->GetSize());
@@ -618,7 +410,7 @@ void ImageVariableDialog::OnShowResponseGraph(wxCommandEvent & e)
         return;
     };
     delete m_popup;
-    Graph graph(300, 200, wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW));
+    wxGraphTools::Graph graph(300, 200, wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW));
     graph.SetChartArea(10, 10, 290, 190);
     graph.SetChartDisplay(0, 0, 1, 1);
     graph.DrawGrid(6, 6);
@@ -668,7 +460,7 @@ void ImageVariableDialog::OnShowResponseGraph(wxCommandEvent & e)
             break;
     };
     //show popup
-    m_popup = new GraphPopupWindow(this, graph.GetGraph());
+    m_popup = new wxGraphTools::GraphPopupWindow(this, graph.GetGraph());
     wxWindow *button = (wxWindow*) e.GetEventObject();
     wxPoint pos=button->ClientToScreen(wxPoint(0,0));
     m_popup->Position(pos, button->GetSize());
