@@ -16,11 +16,14 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
+#ifdef _WIN32
+#include "wx/msw/wrapwin.h"
+#endif
 #include <hugin/PreviewControlPointTool.h>
 #include <panodata/ControlPoint.h>
 
@@ -68,7 +71,7 @@ void PreviewControlPointTool::AfterDrawImagesEvent()
             && helper->GetPanoramaPtr()->getImage(cp.image2Nr).getActive())
         {
             // draw line control points blue instead of orange.
-            bool line = cp.mode != HuginBase::ControlPoint::X_Y;
+            const bool line = cp.mode != HuginBase::ControlPoint::X_Y;
             if (line) 
             {
                 glColor3f(0.0, 0.5, 1.0);
@@ -83,7 +86,7 @@ void PreviewControlPointTool::AfterDrawImagesEvent()
             double x1, y1, x2, y2;
             transforms[cp.image1Nr].transformImgCoord(x1, y1, cp.x1, cp.y1);
             transforms[cp.image2Nr].transformImgCoord(x2, y2, cp.x2, cp.y2);
-            m_greatCircles.drawLineFromSpherical(x1, y1, x2, y2);
+            m_greatCircles.drawLineFromSpherical(x1, y1, x2, y2, 1.0, line);
         }
     }
     glColor3f(1.0, 1.0, 1.0);
