@@ -32,13 +32,17 @@
 #include "hugin_utils/alphanum.h"
 #include "hugin/config_defaults.h"
 #include "wx/mstream.h"
-#include "exiv2/exiv2.hpp"
-#include "exiv2/preview.hpp"
 #ifdef _WIN32
-#include <CommCtrl.h>
+// workaround for a conflict between exiv2 and wxWidgets/CMake built
+#define HAVE_PID_T 1
+#endif
+#include <exiv2/exiv2.hpp>
+#ifdef _WIN32
+#include <commctrl.h>
 #endif
 #include "base_wx/LensTools.h"
 #include "panodata/StandardImageVariableGroups.h"
+#include "base_wx/wxutils.h"
 
 enum
 {
@@ -46,19 +50,7 @@ enum
     ID_SPLIT_PANOS = wxID_HIGHEST + 301
 };
 
-BEGIN_EVENT_TABLE(FindPanoDialog,wxDialog)
-    EVT_BUTTON(XRCID("find_pano_close"), FindPanoDialog::OnButtonClose)
-    EVT_BUTTON(XRCID("find_pano_select_dir"), FindPanoDialog::OnButtonChoose)
-    EVT_BUTTON(XRCID("find_pano_start_stop"), FindPanoDialog::OnButtonStart)
-    EVT_BUTTON(XRCID("find_pano_add_queue"), FindPanoDialog::OnButtonSend)
-    EVT_LISTBOX(XRCID("find_pano_list"), FindPanoDialog::OnSelectPossiblePano)
-    EVT_LIST_ITEM_RIGHT_CLICK(XRCID("find_pano_selected_thumbslist"), FindPanoDialog::OnListItemRightClick)
-    EVT_MENU(ID_REMOVE_IMAGE, FindPanoDialog::OnRemoveImage)
-    EVT_MENU(ID_SPLIT_PANOS, FindPanoDialog::OnSplitPanos)
-    EVT_CLOSE(FindPanoDialog::OnClose)
-END_EVENT_TABLE()
-
-bool SortFilename::operator()(const HuginBase::SrcPanoImage* img1, const HuginBase::SrcPanoImage* img2)
+bool SortFilename::operator()(const HuginBase::SrcPanoImage* img1, const HuginBase::SrcPanoImage* img2) const
 {
     return doj::alphanum_comp(img1->getFilename(),img2->getFilename())<0;
 };
@@ -70,27 +62,26 @@ FindPanoDialog::FindPanoDialog(BatchFrame* batchframe, wxString xrcPrefix)
 {
     // load our children. some children might need special
     // initialization. this will be done later.
-    wxXmlResource::Get()->LoadDialog(this,batchframe,wxT("find_pano_dialog"));
+    wxXmlResource::Get()->LoadDialog(this,batchframe,"find_pano_dialog");
 
-#ifdef __WXMSW__
-    wxIcon myIcon(xrcPrefix+ wxT("data/ptbatcher.ico"),wxBITMAP_TYPE_ICO);
-#else
-    wxIcon myIcon(xrcPrefix + wxT("data/ptbatcher.png"),wxBITMAP_TYPE_PNG);
-#endif
-    SetIcon(myIcon);
     m_batchframe=batchframe;
     m_isRunning=false;
     m_stopped=false;
 
     m_button_start=XRCCTRL(*this,"find_pano_start_stop",wxButton);
+    m_button_start->Bind(wxEVT_BUTTON, &FindPanoDialog::OnButtonStart, this);
     m_button_choose=XRCCTRL(*this,"find_pano_select_dir",wxButton);
+    m_button_choose->Bind(wxEVT_BUTTON, &FindPanoDialog::OnButtonChoose, this);
     m_button_send=XRCCTRL(*this,"find_pano_add_queue",wxButton);
+    m_button_send->Bind(wxEVT_BUTTON, &FindPanoDialog::OnButtonSend, this);
     m_button_close=XRCCTRL(*this,"find_pano_close",wxButton);
+    m_button_close->Bind(wxEVT_BUTTON, &FindPanoDialog::OnButtonClose, this);
     m_textctrl_dir=XRCCTRL(*this,"find_pano_dir",wxTextCtrl);
     m_textctrl_dir->AutoCompleteDirectories();
     m_cb_subdir=XRCCTRL(*this,"find_pano_subdir",wxCheckBox);
     m_statustext=XRCCTRL(*this,"find_pano_label",wxStaticText);
     m_list_pano=XRCCTRL(*this,"find_pano_list",wxCheckListBox);
+    m_list_pano->Bind(wxEVT_LISTBOX, &FindPanoDialog::OnSelectPossiblePano, this);
     m_ch_naming=XRCCTRL(*this,"find_pano_naming",wxChoice);
     m_cb_createLinks=XRCCTRL(*this,"find_pano_create_links",wxCheckBox);
     m_cb_loadDistortion=XRCCTRL(*this,"find_pano_load_distortion",wxCheckBox);
@@ -105,7 +96,7 @@ FindPanoDialog::FindPanoDialog(BatchFrame* batchframe, wxString xrcPrefix)
     // restore position and size
     int dx,dy;
     wxDisplaySize(&dx,&dy);
-    bool maximized = config->Read(wxT("/FindPanoDialog/maximized"), 0l) != 0;
+    bool maximized = config->Read("/FindPanoDialog/maximized", 0l) != 0;
     if (maximized)
     {
         this->Maximize();
@@ -113,8 +104,8 @@ FindPanoDialog::FindPanoDialog(BatchFrame* batchframe, wxString xrcPrefix)
     else
     {
         //size
-        int w = config->Read(wxT("/FindPanoDialog/width"),-1l);
-        int h = config->Read(wxT("/FindPanoDialog/height"),-1l);
+        int w = config->Read("/FindPanoDialog/width",-1l);
+        int h = config->Read("/FindPanoDialog/height",-1l);
         if (w > 0 && w <= dx)
         {
             this->SetClientSize(w,h);
@@ -124,8 +115,8 @@ FindPanoDialog::FindPanoDialog(BatchFrame* batchframe, wxString xrcPrefix)
             this->Fit();
         }
         //position
-        int x = config->Read(wxT("/FindPanoDialog/positionX"),-1l);
-        int y = config->Read(wxT("/FindPanoDialog/positionY"),-1l);
+        int x = config->Read("/FindPanoDialog/positionX",-1l);
+        int y = config->Read("/FindPanoDialog/positionY",-1l);
         if ( y >= 0 && x >= 0 && x < dx && y < dy)
         {
             this->Move(x, y);
@@ -135,42 +126,47 @@ FindPanoDialog::FindPanoDialog(BatchFrame* batchframe, wxString xrcPrefix)
             this->Move(0, 44);
         }
     }
-    long splitterPos = config->Read(wxT("/FindPanoDialog/splitterPos"), -1l);
+    long splitterPos = config->Read("/FindPanoDialog/splitterPos", -1l);
     if (splitterPos != -1)
     {
         XRCCTRL(*this, "find_pano_splitter", wxSplitterWindow)->SetSashPosition(splitterPos);
     };
-    wxString path=config->Read(wxT("/FindPanoDialog/actualPath"),wxEmptyString);
+    wxString path=config->Read("/FindPanoDialog/actualPath",wxEmptyString);
     if(!path.IsEmpty())
     {
         m_textctrl_dir->SetValue(path);
     }
     bool val;
-    config->Read(wxT("/FindPanoDialog/includeSubDirs"),&val,false);
+    config->Read("/FindPanoDialog/includeSubDirs",&val,false);
     m_cb_subdir->SetValue(val);
-    long i=config->Read(wxT("/FindPanoDialog/Naming"),0l);
+    long i=config->Read("/FindPanoDialog/Naming",0l);
     m_ch_naming->SetSelection(i);
-    config->Read(wxT("/FindPanoDialog/linkStacks"),&val,true);
+    config->Read("/FindPanoDialog/linkStacks",&val,true);
     m_cb_createLinks->SetValue(val);
-    config->Read(wxT("/FindPanoDialog/loadDistortion"),&val,false);
+    config->Read("/FindPanoDialog/loadDistortion",&val,false);
     m_cb_loadDistortion->SetValue(val);
-    config->Read(wxT("/FindPanoDialog/loadVignetting"),&val,false);
+    config->Read("/FindPanoDialog/loadVignetting",&val,false);
     m_cb_loadVignetting->SetValue(val);
-    i=config->Read(wxT("/FindPanoDialog/MinNumberImages"), 2l);
+    i=config->Read("/FindPanoDialog/MinNumberImages", 2l);
     m_sc_minNumberImages->SetValue(i);
-    i=config->Read(wxT("/FindPanoDialog/MaxTimeDiff"), 30l);
+    i=config->Read("/FindPanoDialog/MaxTimeDiff", 30l);
     m_sc_maxTimeDiff->SetValue(i);
-    i = config->Read(wxT("/FindPanoDialog/DefaultBlender"), static_cast<long>(HuginBase::PanoramaOptions::ENBLEND_BLEND));
+    i = config->Read("/FindPanoDialog/DefaultBlender", static_cast<long>(HuginBase::PanoramaOptions::ENBLEND_BLEND));
     SelectListValue(m_ch_blender, i);
     m_button_send->Disable();
     m_thumbs = new wxImageList(THUMBSIZE, THUMBSIZE, true, 0);
     m_thumbsList = XRCCTRL(*this, "find_pano_selected_thumbslist", wxListCtrl);
     m_thumbsList->SetImageList(m_thumbs, wxIMAGE_LIST_NORMAL);
+    m_thumbsList->Bind(wxEVT_MOTION, &FindPanoDialog::OnListMouseMove, this);
+    m_thumbsList->Bind(wxEVT_LIST_ITEM_RIGHT_CLICK, &FindPanoDialog::OnListItemRightClick, this);
 #ifdef _WIN32
     // default image spacing is too big, wxWidgets does not provide direct 
     // access to the spacing, so using the direct API function
     ListView_SetIconSpacing(m_thumbsList->GetHandle(), THUMBSIZE + 20, THUMBSIZE + 20);
 #endif
+    Bind(wxEVT_MENU, &FindPanoDialog::OnRemoveImage, this, ID_REMOVE_IMAGE);
+    Bind(wxEVT_MENU, &FindPanoDialog::OnSplitPanos, this, ID_SPLIT_PANOS);
+    Bind(wxEVT_CLOSE_WINDOW, &FindPanoDialog::OnClose, this);
 };
 
 FindPanoDialog::~FindPanoDialog()
@@ -179,40 +175,37 @@ FindPanoDialog::~FindPanoDialog()
     if(!this->IsMaximized())
     {
         wxSize sz = this->GetClientSize();
-        config->Write(wxT("/FindPanoDialog/width"), sz.GetWidth());
-        config->Write(wxT("/FindPanoDialog/height"), sz.GetHeight());
+        config->Write("/FindPanoDialog/width", sz.GetWidth());
+        config->Write("/FindPanoDialog/height", sz.GetHeight());
         wxPoint ps = this->GetPosition();
-        config->Write(wxT("/FindPanoDialog/positionX"), ps.x);
-        config->Write(wxT("/FindPanoDialog/positionY"), ps.y);
-        config->Write(wxT("/FindPanoDialog/maximized"), 0);
+        config->Write("/FindPanoDialog/positionX", ps.x);
+        config->Write("/FindPanoDialog/positionY", ps.y);
+        config->Write("/FindPanoDialog/maximized", 0);
     }
     else
     {
-        config->Write(wxT("/FindPanoDialog/maximized"), 1l);
+        config->Write("/FindPanoDialog/maximized", 1l);
     };
-    config->Write(wxT("/FindPanoDialog/splitterPos"), XRCCTRL(*this, "find_pano_splitter", wxSplitterWindow)->GetSashPosition());
-    config->Write(wxT("/FindPanoDialog/actualPath"),m_textctrl_dir->GetValue());
-    config->Write(wxT("/FindPanoDialog/includeSubDirs"),m_cb_subdir->GetValue());
-    config->Write(wxT("/FindPanoDialog/Naming"),m_ch_naming->GetSelection());
-    config->Write(wxT("/FindPanoDialog/linkStacks"),m_cb_createLinks->GetValue());
-    config->Write(wxT("/FindPanoDialog/loadDistortion"),m_cb_loadDistortion->GetValue());
-    config->Write(wxT("/FindPanoDialog/loadVignetting"),m_cb_loadDistortion->GetValue());
-    config->Write(wxT("/FindPanoDialog/MinNumberImages"), m_sc_minNumberImages->GetValue());
-    config->Write(wxT("/FindPanoDialog/MaxTimeDiff"), m_sc_maxTimeDiff->GetValue());
-    config->Write(wxT("/FindPanoDialog/DefaultBlender"), static_cast<long>(GetSelectedValue(m_ch_blender)));
+    config->Write("/FindPanoDialog/splitterPos", XRCCTRL(*this, "find_pano_splitter", wxSplitterWindow)->GetSashPosition());
+    config->Write("/FindPanoDialog/actualPath",m_textctrl_dir->GetValue());
+    config->Write("/FindPanoDialog/includeSubDirs",m_cb_subdir->GetValue());
+    config->Write("/FindPanoDialog/Naming",m_ch_naming->GetSelection());
+    config->Write("/FindPanoDialog/linkStacks",m_cb_createLinks->GetValue());
+    config->Write("/FindPanoDialog/loadDistortion",m_cb_loadDistortion->GetValue());
+    config->Write("/FindPanoDialog/loadVignetting",m_cb_loadDistortion->GetValue());
+    config->Write("/FindPanoDialog/MinNumberImages", m_sc_minNumberImages->GetValue());
+    config->Write("/FindPanoDialog/MaxTimeDiff", m_sc_maxTimeDiff->GetValue());
+    config->Write("/FindPanoDialog/DefaultBlender", static_cast<long>(GetSelectedValue(m_ch_blender)));
     CleanUpPanolist();
     delete m_thumbs;
 };
 
 void FindPanoDialog::CleanUpPanolist()
 {
-    if(m_panos.size()>0)
+    while(!m_panos.empty())
     {
-        while(!m_panos.empty())
-        {
-            delete m_panos.back();
-            m_panos.pop_back();
-        };
+        delete m_panos.back();
+        m_panos.pop_back();
     };
 };
 
@@ -232,10 +225,10 @@ void FindPanoDialog::OnClose(wxCloseEvent& e)
 
 void FindPanoDialog::OnButtonClose(wxCommandEvent& e)
 {
-    if(m_panos.size()>0)
+    if(!m_panos.empty())
     {
-        if(wxMessageBox(_("The list contains possibly unprocessed panoramas.\nIf you close the dialog, you will lose them.\nContinue anyway?"),
-                        _("Question"),wxYES_NO|wxICON_QUESTION,this)==wxNO)
+        if (hugin_utils::HuginMessageBox(_("The list contains possibly unprocessed panoramas.\nIf you close the dialog, you will lose them.\nContinue anyway?"),
+            _("PTBatcherGUI"), wxYES_NO | wxICON_WARNING, this) == wxNO)
         {
             return;
         };
@@ -267,10 +260,10 @@ void FindPanoDialog::OnButtonStart(wxCommandEvent& e)
         m_start_dir=m_textctrl_dir->GetValue();
         if(wxDir::Exists(m_start_dir))
         {
-            if(m_panos.size()>0)
+            if(!m_panos.empty())
             {
-                if(wxMessageBox(_("The list contains still not yet processed panoramas.\nIf you continue, they will be disregarded.\nDo you still want to continue?"),
-                                _("Question"),wxYES_NO|wxICON_QUESTION,this)==wxNO)
+                if (hugin_utils::HuginMessageBox(_("The list contains still not yet processed panoramas.\nIf you continue, they will be disregarded.\nDo you still want to continue?"),
+                    _("PTBatcherGUI"), wxYES_NO | wxICON_WARNING, this) == wxNO)
                 {
                     return;
                 };
@@ -290,15 +283,15 @@ void FindPanoDialog::OnButtonStart(wxCommandEvent& e)
         }
         else
         {
-            wxMessageBox(wxString::Format(_("Directory %s does not exist.\nPlease give an existing directory."),m_start_dir.c_str()),
-                         _("Warning"),wxOK | wxICON_EXCLAMATION,this);
+            hugin_utils::HuginMessageBox(wxString::Format(_("Directory %s does not exist.\nPlease give an existing directory."), m_start_dir),
+                _("PTBatcherGUI"), wxOK | wxICON_EXCLAMATION, this);
         };
     };
 }
 
 void FindPanoDialog::OnButtonSend(wxCommandEvent& e)
 {
-    if(m_panos.size()==0)
+    if(m_panos.empty())
     {
         return;
     }
@@ -312,7 +305,7 @@ void FindPanoDialog::OnButtonSend(wxCommandEvent& e)
     };
     if(nr==0)
     {
-        wxMessageBox(_("You have selected no possible panorama.\nPlease select at least one panorama and try again."),_("Warning"),wxOK|wxICON_EXCLAMATION,this);
+        hugin_utils::HuginMessageBox(_("You have not selected a panorama.\nPlease select at least one panorama and try again."), _("PTBatcherGUI"), wxOK | wxICON_EXCLAMATION, this);
         return;
     }
     bool failed=false;
@@ -335,7 +328,7 @@ void FindPanoDialog::OnButtonSend(wxCommandEvent& e)
     };
     if(failed)
     {
-        wxMessageBox(_("Not all project files could be written successfully.\nMaybe you have no write permission for these directories or your disc is full."),_("Error"),wxOK,this);
+        hugin_utils::HuginMessageBox(_("Not all project files could be written successfully.\nMaybe you have no write permission for these directories or your disc is full."), _("PTBatcherGUI"), wxOK, this);
     };
     this->Close();
 };
@@ -359,8 +352,8 @@ void FindPanoDialog::OnSelectPossiblePano(wxCommandEvent &e)
         XRCCTRL(*this, "find_pano_selected_cam", wxStaticText)->SetLabel(m_panos[selected]->GetCameraName());
         XRCCTRL(*this, "find_pano_selected_lens", wxStaticText)->SetLabel(m_panos[selected]->GetLensName());
         XRCCTRL(*this, "find_pano_selected_focallength", wxStaticText)->SetLabel(m_panos[selected]->GetFocalLength());
-        XRCCTRL(*this, "find_pano_selected_date_time", wxStaticText)->SetLabel(m_panos[selected]->GetStartString() + wxT(" (")+ m_panos[selected]->GetDuration() + wxT(")"));
-        m_panos[selected]->PopulateListCtrl(m_thumbsList, m_thumbs);
+        XRCCTRL(*this, "find_pano_selected_date_time", wxStaticText)->SetLabel(m_panos[selected]->GetStartString() + " ("+ m_panos[selected]->GetDuration() + ")");
+        m_panos[selected]->PopulateListCtrl(m_thumbsList, m_thumbs, m_tooltips);
     }
     else
     {
@@ -370,6 +363,7 @@ void FindPanoDialog::OnSelectPossiblePano(wxCommandEvent &e)
         XRCCTRL(*this, "find_pano_selected_date_time", wxStaticText)->SetLabel(wxEmptyString);
         m_thumbsList->DeleteAllItems();
         m_thumbs->RemoveAll();
+        m_tooltips.Clear();
     };
 };
 
@@ -455,6 +449,24 @@ void FindPanoDialog::OnSplitPanos(wxCommandEvent &e)
     };
 }
 
+void FindPanoDialog::OnListMouseMove(wxMouseEvent & e)
+{
+    if (m_tooltips.IsEmpty())
+    {
+        return;
+    }
+    int flags;
+    long item = m_thumbsList->HitTest(e.GetPosition(), flags);
+    if (item != wxNOT_FOUND && (flags&wxLIST_HITTEST_ONITEM) && item < m_tooltips.GetCount())
+    {
+        m_thumbsList->SetToolTip(m_tooltips[item]);
+    }
+    else
+    {
+        m_thumbsList->UnsetToolTip();
+    };
+};
+
 int SortWxFilenames(const wxString& s1,const wxString& s2)
 {
     return doj::alphanum_comp(std::string(s1.mb_str(wxConvLocal)),std::string(s2.mb_str(wxConvLocal)));
@@ -475,8 +487,8 @@ void FindPanoDialog::SearchInDir(wxString dirstring, const bool includeSubdir, c
         wxFileName file(fileList[j]);
         file.MakeAbsolute();
         wxString ext=file.GetExt();
-        if(ext.CmpNoCase(wxT("jpg"))==0 || ext.CmpNoCase(wxT("jpeg"))==0 ||
-                ext.CmpNoCase(wxT("tif"))==0 || ext.CmpNoCase(wxT("tiff"))==0)
+        if(ext.CmpNoCase("jpg")==0 || ext.CmpNoCase("jpeg")==0 ||
+                ext.CmpNoCase("tif")==0 || ext.CmpNoCase("tiff")==0)
         {
             std::string filenamestr(file.GetFullPath().mb_str(HUGIN_CONV_FILENAME));
             HuginBase::SrcPanoImage* img = new HuginBase::SrcPanoImage;
@@ -497,7 +509,7 @@ void FindPanoDialog::SearchInDir(wxString dirstring, const bool includeSubdir, c
             if(!img->getExifMake().empty() && !img->getExifModel().empty() && 
                 img->getExifFocalLength()!=0 && img->getCropFactor()!=0)
             {
-                img->readProjectionFromDB();
+                img->readProjectionFromDB(false);
                 if(loadDistortion)
                 {
                     img->readDistortionFromDB();
@@ -536,7 +548,7 @@ void FindPanoDialog::SearchInDir(wxString dirstring, const bool includeSubdir, c
         //allow processing events
         wxGetApp().Yield(true);
     };
-    if(!m_stopped && newPanos.size()>0)
+    if(!m_stopped && !newPanos.empty())
     {
         for(size_t i=0; i<newPanos.size(); i++)
         {
@@ -571,14 +583,14 @@ void FindPanoDialog::SearchInDir(wxString dirstring, const bool includeSubdir, c
         m_button_start->SetLabel(_("Start"));
         EnableButtons(true);
         //enable send button if at least one panorama found
-        m_button_send->Enable(m_panos.size()>0);
-        if(m_panos.size()>0)
+        m_button_send->Enable(!m_panos.empty());
+        if(!m_panos.empty())
         {
-            m_statustext->SetLabel(wxString::Format(_("Found %d possible panoramas."), static_cast<int>(m_panos.size())));
+            m_statustext->SetLabel(wxString::Format(_("Found %d possible panoramas"), static_cast<int>(m_panos.size())));
         }
         else
         {
-            m_statustext->SetLabel(_("No possible panoramas found."));
+            m_statustext->SetLabel(_("No possible panoramas found"));
         };
         TIFFSetWarningHandler(m_oldtiffwarning);
     };
@@ -613,6 +625,10 @@ bool PossiblePano::BelongsTo(HuginBase::SrcPanoImage* img, const wxTimeSpan max_
     {
         return false;
     }
+    if (fabs(m_cropfactor - img->getCropFactor()) > 0.01)
+    {
+        return false;
+    }
     if(m_size!=img->getSize())
     {
         return false;
@@ -647,6 +663,7 @@ void PossiblePano::AddSrcPanoImage(HuginBase::SrcPanoImage* img)
         m_camera=img->getExifModel();
         m_lens=img->getExifLens();
         m_focallength=img->getExifFocalLength();
+        m_cropfactor = img->getCropFactor();
         m_size=img->getSize();
         m_dt_start=GetDateTime(img);
         m_dt_end=m_dt_start;
@@ -675,11 +692,11 @@ const wxString PossiblePano::GetFilestring(const wxString BasePath, const bool s
     wxFileName f2(wxString((*rit)->getFilename().c_str(),HUGIN_CONV_FILENAME));
     if(stripExtension)
     {
-        return f1.GetPath(wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR)+f1.GetName()+wxT("-")+f2.GetName();
+        return f1.GetPath(wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR)+f1.GetName()+"-"+f2.GetName();
     }
     else
     {
-        return f1.GetFullPath()+wxT(" - ")+f2.GetFullName();
+        return f1.GetFullPath()+" - "+f2.GetFullName();
     };
 };
 
@@ -693,8 +710,8 @@ bool PossiblePano::GetNewProjectFilename(NamingConvention nc,const wxString base
     wxString mask;
     unsigned int i=1;
     projectFile.SetPath(basePath);
-    projectFile.SetName(wxT("pano"));
-    projectFile.SetExt(wxT("pto"));
+    projectFile.SetName("pano");
+    projectFile.SetExt("pto");
     if(!projectFile.IsDirWritable())
     {
         return false;
@@ -702,7 +719,7 @@ bool PossiblePano::GetNewProjectFilename(NamingConvention nc,const wxString base
     switch(nc)
     {
         case NAMING_PANO:
-            mask=wxT("panorama%d");
+            mask="panorama%d";
             break;
         case NAMING_FIRST_LAST:
             mask=GetFilestring(basePath,true);
@@ -711,7 +728,7 @@ bool PossiblePano::GetNewProjectFilename(NamingConvention nc,const wxString base
             {
                 return true;
             };
-            mask=mask+wxT("_%d");
+            mask=mask+"_%d";
             break;
         case NAMING_FOLDER:
             {
@@ -726,7 +743,7 @@ bool PossiblePano::GetNewProjectFilename(NamingConvention nc,const wxString base
                 {
                     return true;
                 }
-                mask=mask+wxT("_%d");
+                mask=mask+"_%d";
             }
             break;
         case NAMING_TEMPLATE:
@@ -741,11 +758,11 @@ bool PossiblePano::GetNewProjectFilename(NamingConvention nc,const wxString base
                 {
                     return true;
                 }
-                mask=mask+wxT("_%d");
+                mask=mask+"_%d";
             };
             break;
         default:
-            mask=wxT("panorama%d");
+            mask="panorama%d";
     };
 
     projectFile.SetName(wxString::Format(mask,i));
@@ -777,9 +794,21 @@ wxString PossiblePano::GeneratePanorama(NamingConvention nc, bool createLinks, H
         return wxEmptyString;
     };
     //generate panorama
+    double redBalanceAnchor = (*m_images.begin())->getExifRedBalance();
+    double blueBalanceAnchor = (*m_images.begin())->getExifBlueBalance();
+    if (fabs(redBalanceAnchor)<1e-2)
+    {
+        redBalanceAnchor = 1;
+    };
+    if (fabs(blueBalanceAnchor)<1e-2)
+    {
+        blueBalanceAnchor = 1;
+    };
     HuginBase::Panorama pano;
     for(ImageSet::iterator it=m_images.begin(); it!=m_images.end(); ++it)
     {
+        (*it)->setWhiteBalanceRed((*it)->getExifRedBalance() / redBalanceAnchor);
+        (*it)->setWhiteBalanceBlue((*it)->getExifBlueBalance() / blueBalanceAnchor);
         pano.addImage(*(*it));
     };
     //assign all images the same lens number
@@ -787,28 +816,9 @@ wxString PossiblePano::GeneratePanorama(NamingConvention nc, bool createLinks, H
     HuginBase::ImageVariableGroup& lenses = variable_groups.getLenses();
     if(pano.getNrOfImages()>1)
     {
-        double redBalanceAnchor=pano.getImage(pano.getOptions().colorReferenceImage).getExifRedBalance();
-        double blueBalanceAnchor=pano.getImage(pano.getOptions().colorReferenceImage).getExifBlueBalance();
-        if(fabs(redBalanceAnchor)<1e-2)
-        {
-            redBalanceAnchor=1;
-        };
-        if(fabs(blueBalanceAnchor)<1e-2)
-        {
-            blueBalanceAnchor=1;
-        };
         for(unsigned int i=1; i<pano.getNrOfImages(); i++)
         {
-            HuginBase::SrcPanoImage img = pano.getSrcImage(i);
-            double ev=img.getExposureValue();
-            lenses.switchParts(i,lenses.getPartNumber(0));
-            lenses.unlinkVariableImage(HuginBase::ImageVariableGroup::IVE_ExposureValue, i);
-            img.setExposureValue(ev);
-            lenses.unlinkVariableImage(HuginBase::ImageVariableGroup::IVE_WhiteBalanceRed, i);
-            lenses.unlinkVariableImage(HuginBase::ImageVariableGroup::IVE_WhiteBalanceBlue, i);
-            img.setWhiteBalanceRed(img.getExifRedBalance()/redBalanceAnchor);
-            img.setWhiteBalanceBlue(img.getExifBlueBalance()/blueBalanceAnchor);
-            pano.setSrcImage(i, img);
+            lenses.switchParts(i, lenses.getPartNumber(0));
         };
     };
     if (pano.hasPossibleStacks())
@@ -820,8 +830,8 @@ wxString PossiblePano::GeneratePanorama(NamingConvention nc, bool createLinks, H
     //set default exposure value
     opts.outputExposureValue = pano.getImage(0).getExposureValue();
     wxConfigBase* config = wxConfigBase::Get();
-    opts.quality = config->Read(wxT("/output/jpeg_quality"),HUGIN_JPEG_QUALITY);
-    switch(config->Read(wxT("/output/tiff_compression"), HUGIN_TIFF_COMPRESSION))
+    opts.quality = config->Read("/output/jpeg_quality",HUGIN_JPEG_QUALITY);
+    switch(config->Read("/output/tiff_compression", HUGIN_TIFF_COMPRESSION))
     {
         case 0:
         default:
@@ -841,7 +851,7 @@ wxString PossiblePano::GeneratePanorama(NamingConvention nc, bool createLinks, H
             opts.tiffCompression = "DEFLATE";
             break;
     }
-    switch (config->Read(wxT("/output/ldr_format"), HUGIN_LDR_OUTPUT_FORMAT))
+    switch (config->Read("/output/ldr_format", HUGIN_LDR_OUTPUT_FORMAT))
     {
         case 1:
             opts.outputImageType ="jpg";
@@ -859,37 +869,27 @@ wxString PossiblePano::GeneratePanorama(NamingConvention nc, bool createLinks, H
     }
     opts.outputFormat = HuginBase::PanoramaOptions::TIFF_m;
     opts.blendMode = defaultBlender;
-    opts.enblendOptions = config->Read(wxT("Enblend/Args"),wxT(HUGIN_ENBLEND_ARGS)).mb_str(wxConvLocal);
-    opts.enfuseOptions = config->Read(wxT("Enfuse/Args"),wxT(HUGIN_ENFUSE_ARGS)).mb_str(wxConvLocal);
-    opts.interpolator = (vigra_ext::Interpolator)config->Read(wxT("Nona/Interpolator"),HUGIN_NONA_INTERPOLATOR);
-    opts.tiff_saveROI = config->Read(wxT("Nona/CroppedImages"),HUGIN_NONA_CROPPEDIMAGES)!=0;
+    opts.enblendOptions = config->Read("Enblend/Args",HUGIN_ENBLEND_ARGS).mb_str(wxConvLocal);
+    opts.enfuseOptions = config->Read("Enfuse/Args",HUGIN_ENFUSE_ARGS).mb_str(wxConvLocal);
+    opts.interpolator = (vigra_ext::Interpolator)config->Read("Nona/Interpolator",HUGIN_NONA_INTERPOLATOR);
+    opts.tiff_saveROI = config->Read("Nona/CroppedImages",HUGIN_NONA_CROPPEDIMAGES)!=0;
     opts.hdrMergeMode = HuginBase::PanoramaOptions::HDRMERGE_AVERAGE;
     opts.hdrmergeOptions = HUGIN_HDRMERGE_ARGS;
-    opts.verdandiOptions = config->Read(wxT("/VerdandiDefaultArgs"), wxEmptyString).mb_str(wxConvLocal);
+    opts.verdandiOptions = config->Read("/VerdandiDefaultArgs", wxEmptyString).mb_str(wxConvLocal);
     pano.setOptions(opts);
     // set optimizer switches
     pano.setOptimizerSwitch(HuginBase::OPT_POSITION);
     pano.setPhotometricOptimizerSwitch(HuginBase::OPT_EXPOSURE | HuginBase::OPT_VIGNETTING | HuginBase::OPT_RESPONSE);
 
-    std::ofstream script(projectFile.GetFullPath().mb_str(HUGIN_CONV_FILENAME));
-    script.exceptions ( std::ofstream::eofbit | std::ofstream::failbit | std::ofstream::badbit );
-    if(!script.good())
+    if (pano.WritePTOFile(std::string(projectFile.GetFullPath().mb_str(HUGIN_CONV_FILENAME)),
+        std::string(projectFile.GetPath(wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR).mb_str(HUGIN_CONV_FILENAME))))
     {
-        return wxEmptyString;
-    };
-    HuginBase::UIntSet all;
-    fill_set(all, 0, pano.getNrOfImages()-1);
-    try
-    {
-        std::string Pathprefix(projectFile.GetPath(wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR).mb_str(HUGIN_CONV_FILENAME));
-        pano.printPanoramaScript(script, pano.getOptimizeVector(), pano.getOptions(), all, false, Pathprefix);
+        return projectFile.GetFullPath();
     }
-    catch (...)
+    else
     {
         return wxEmptyString;
     };
-    script.close();
-    return projectFile.GetFullPath();
 };
 
 wxString PossiblePano::GetCameraName()
@@ -904,7 +904,11 @@ wxString PossiblePano::GetLensName()
 
 wxString PossiblePano::GetFocalLength()
 {
-    return wxString::Format(wxT("%0.1f mm"), m_focallength);
+    if (!m_images.empty())
+    {
+        return FormatString::GetFocalLength(*m_images.begin());
+    };
+    return wxString::Format("%0.1f mm", m_focallength);
 };
 
 wxString PossiblePano::GetStartString()
@@ -914,25 +918,22 @@ wxString PossiblePano::GetStartString()
 
 wxString PossiblePano::GetDuration()
 {
-    wxTimeSpan diff = m_dt_end.Subtract(m_dt_start);
-    if (diff.GetSeconds() > 60)
-    {
-        return diff.Format(_("%M:%S min"));
-    }
-    else
-    {
-        return diff.Format(_("%S s"));
-    };
+    return hugin_utils::GetFormattedTimeSpan(m_dt_end.Subtract(m_dt_start));
 };
 
-void PossiblePano::PopulateListCtrl(wxListCtrl* list, wxImageList* thumbs)
+void PossiblePano::PopulateListCtrl(wxListCtrl* list, wxImageList* thumbs, wxArrayString& tooltips)
 {
     list->DeleteAllItems();
     thumbs->RemoveAll();
+    tooltips.Clear();
     wxBusyCursor cursor;
     for (ImageSet::iterator it = m_images.begin(); it != m_images.end(); ++it)
     {
+#if defined EXIV2_VERSION && EXIV2_TEST_VERSION(0,27,99)
+        Exiv2::Image::UniquePtr image;
+#else
         Exiv2::Image::AutoPtr image;
+#endif
         bool opened = false;
         try
         {
@@ -942,11 +943,20 @@ void PossiblePano::PopulateListCtrl(wxListCtrl* list, wxImageList* thumbs)
         catch (...)
         {
             std::cerr << __FILE__ << " " << __LINE__ << " Error opening file" << std::endl;
+            continue;
         }
         int index = -1;
         if (opened)
         {
-            image->readMetadata();
+            try
+            {
+                image->readMetadata();
+            }
+            catch (const Exiv2::Error& e)
+            {
+                std::cerr << __FILE__ << " " << __LINE__ << " Exiv2: Error reading metadata (" << e.what() << ")" << std::endl;
+                continue;
+            }
             // read all thumbnails
             Exiv2::PreviewManager previews(*image);
             Exiv2::PreviewPropertiesList lists = previews.getPreviewProperties();
@@ -981,6 +991,27 @@ void PossiblePano::PopulateListCtrl(wxListCtrl* list, wxImageList* thumbs)
                     x = (THUMBSIZE - newWidth) / 2;
                     y = 0;
                 }
+                // rotate according to orientation tag
+                if ((*it)->getRoll() == 90)
+                {
+                    rawImage = rawImage.Rotate90();
+                    std::swap(x, y);
+                }
+                else
+                {
+                    if ((*it)->getRoll() == 270)
+                    {
+                        rawImage = rawImage.Rotate90(false);
+                        std::swap(x, y);
+                    }
+                    else
+                    {
+                        if ((*it)->getRoll() == 180)
+                        {
+                            rawImage = rawImage.Rotate180();
+                        };
+                    };
+                };
                 // create final bitmap with centered thumbnail
                 wxBitmap bitmap(THUMBSIZE, THUMBSIZE);
                 wxMemoryDC dc(bitmap);
@@ -999,6 +1030,10 @@ void PossiblePano::PopulateListCtrl(wxListCtrl* list, wxImageList* thumbs)
         // create item in thumb list
         wxFileName fn(wxString((*it)->getFilename().c_str(), HUGIN_CONV_FILENAME));
         list->InsertItem(list->GetItemCount(), fn.GetFullName(), index);
+        // build tooltip text
+        tooltips.Add(fn.GetFullName() + "\n" + FormatString::GetAperture(*it) + ", "
+            + FormatString::GetExposureTime(*it) + ", " + _("ISO") + FormatString::GetIso(*it)
+            + "\n" + FormatString::GetExifDateTime(*it));
     };
 };
 
