@@ -23,13 +23,14 @@
  */
 
 #include "utils.h"
+#include "stl_utils.h"
 #include "hugin_version.h"
 #include "hugin_config.h"
 
 #ifdef _WIN32
     #define NOMINMAX
     #include <sys/utime.h>
-    #include <ShlObj.h>
+    #include <shlobj.h>
 #else
     #include <sys/time.h>
     #include <cstdlib>
@@ -48,14 +49,7 @@
 #include <hugin_config.h>
 #endif
 #include <algorithm>
-#ifdef HAVE_STD_FILESYSTEM
 #include <filesystem>
-namespace fs = std::tr2::sys;
-#else
-#define BOOST_FILESYSTEM_VERSION 3
-#include <boost/filesystem.hpp>
-namespace fs = boost::filesystem;
-#endif
 #include <lcms2.h>
 
 #ifdef __APPLE__
@@ -64,10 +58,18 @@ namespace fs = boost::filesystem;
 #include <libgen.h>       /* dirname */
 #endif
 
+#if defined HAVE_EPOXY && HAVE_EPOXY
+#include <epoxy/gl.h>
+#ifdef _WIN32
+#include <epoxy/wgl.h>
+#endif
+#else
 #include <GL/glew.h>
 #ifdef _WIN32
 #include <GL/wglew.h>
-#elif defined __APPLE__
+#endif
+#endif
+#if defined __APPLE__
   #include <GLUT/glut.h>
 #endif
 
@@ -229,7 +231,8 @@ std::string doubleToString(double d, int digits)
     if (digits < 0) {
         strcpy(fmt,"%f");
     } else {
-        std::sprintf(fmt,"%%.%df",digits);
+        // max. 16 digits to prevent overflow
+        std::sprintf(fmt, "%%.%df", std::min(digits, 16));
     }
     char c[1024];
     c[1023] = 0;
@@ -329,6 +332,12 @@ void ReplaceAll(std::string& s, const std::string& oldChar, char newChar)
     };
 };
 
+bool StringContainsCaseInsensitive(const std::string& s1, const std::string& s2)
+{
+    const auto it = std::search(s1.begin(), s1.end(), s2.begin(), s2.end(), [](const char a, const char b)->bool { return std::tolower(a) == std::tolower(b); });
+    return it != s1.end();
+}
+
     void ControlPointErrorColour(const double cperr, 
         double &r,double &g, double &b)
     {
@@ -393,28 +402,54 @@ std::string GetAbsoluteFilename(const std::string& filename)
 #endif
 };
 
+bool IsFileTypeSupported(const std::string& filename)
+{
+    const std::string extension = tolower(getExtension(filename));
+    return (vigra::impexListExtensions().find(extension) != std::string::npos);
+};
+
+void EnforceExtension(std::string& filename, const std::string& defaultExtension)
+{
+    const std::string extension = getExtension(filename);
+    if (extension.empty())
+    {
+        filename = stripExtension(filename) + "." + defaultExtension;
+    };
+}
+
+std::string GetOutputFilename(const std::string& out, const std::string& in, const std::string& suffix)
+{
+    if (out.empty())
+    {
+        const std::string extension = getExtension(in);
+        if (extension.empty())
+        {
+            return in + "_" + suffix;
+        }
+        else
+        {
+            return in.substr(0, in.length() - extension.length() - 1).append("_" + suffix + "." + extension);
+        };
+    }
+    else
+    {
+        return out;
+    };
+}
+
+
 std::string GetDataDir()
 {
 #ifdef _WIN32
     char buffer[MAX_PATH];//always use MAX_PATH for filepaths
     GetModuleFileName(NULL,buffer,sizeof(buffer));
-    std::string working_path(buffer);
-    std::string data_path("");
-    //remove filename
-    std::string::size_type pos=working_path.rfind("\\");
-    if(pos!=std::string::npos)
+    std::filesystem::path data_path(buffer);
+    data_path.remove_filename();
+    if (data_path.has_parent_path())
     {
-        working_path.erase(pos);
-        //remove last dir: should be bin
-        pos=working_path.rfind("\\");
-        if(pos!=std::string::npos)
-        {
-            working_path.erase(pos);
-            //append path delimiter and path
-            working_path.append("\\share\\hugin\\data\\");
-            data_path=working_path;
-        }
-    }
+        return std::filesystem::absolute(data_path.parent_path() / ".." / "share" / "hugin" / "data").string() + "\\";
+    };
+    return std::string();
 #elif defined MAC_SELF_CONTAINED_BUNDLE
     char path[PATH_MAX + 1];
     uint32_t size = sizeof(path);
@@ -424,49 +459,79 @@ std::string GetDataDir()
         data_path=dirname(path);
         data_path.append("/../Resources/xrc/");
     }
-#else
-    std::string data_path = (INSTALL_DATA_DIR);
-#endif
     return data_path;
+#elif defined UNIX_SELF_CONTAINED_BUNDLE
+    std::filesystem::path data_path = std::filesystem::read_symlink("/proc/self/exe");
+    data_path.remove_filename();
+    if (data_path.has_parent_path())
+    {
+        return (data_path.parent_path() / "share/hugin/data").string() + "/";
+    };
+    return std::string();
+#else
+    return std::string(INSTALL_DATA_DIR);
+#endif
 };
 
+#ifndef _WIN32
+std::string GetHomeDir()
+{
+    char *homedir = getenv("HOME");
+    struct passwd *pw;
+    if (homedir == NULL)
+    {
+        pw = getpwuid(getuid());
+        if (pw != NULL)
+        {
+            homedir = pw->pw_dir;
+        };
+    };
+    if (homedir == NULL)
+    {
+        return std::string();
+    };
+    return std::string(homedir);
+}
+#endif
+
 std::string GetUserAppDataDir()
 {
-    fs::path path;
+    std::filesystem::path path;
 #ifdef _WIN32
     char fullpath[_MAX_PATH];
     if(SHGetFolderPath(NULL, CSIDL_APPDATA | CSIDL_FLAG_CREATE, NULL, 0, fullpath)!=S_OK)
     {
         return std::string();
     };
-    path = fs::path(fullpath);
+    path = std::filesystem::path(fullpath);
     path /= "hugin";
 #else
-    char *homedir = getenv("HOME");
-    struct passwd *pw;
-    if (homedir == NULL)
+    char *xdgDataDir = getenv("XDG_DATA_HOME");
+    if (xdgDataDir == NULL || strlen(xdgDataDir) == 0)
     {
-        pw = getpwuid(getuid());
-        if(pw != NULL)
+        // no XDG_DATA_HOME enviroment variable set or empty variable
+        // use $HOME/.local/share instead
+        const  std::string homeDir = GetHomeDir();
+        if (homeDir.empty())
         {
-            homedir = pw->pw_dir;
+            return std::string();
         };
-    };
-    if(homedir == NULL)
+        path = std::filesystem::path(homeDir);
+        path /= ".local/share/hugin";
+    }
+    else
     {
-        return std::string();
+        // XDG_DATA_HOME set, use hugin sub directory
+        path = std::filesystem::path(xdgDataDir);
+        path /= "hugin";
     };
-    path = fs::path(homedir);
-    // we have already a file with name ".hugin" for our wxWidgets settings
-    // therefore we use directory ".hugindata" in homedir
-    path /= ".hugindata";
 #endif
-    if (!fs::exists(path))
+    if (!std::filesystem::exists(path))
     {
-        if (!fs::create_directories(path))
+        if (!std::filesystem::create_directories(path))
         {
             std::cerr << "ERROR: Could not create destination directory: " << path.string() << std::endl
-                << "Maybe you have not sufficent rights to create this directory." << std::endl;
+                << "Maybe you have not sufficient rights to create this directory." << std::endl;
             return std::string();
         };
     };
@@ -542,6 +607,9 @@ bool CreateContext(int *argcp, char **argv)
     {
         return false;
     };
+#if defined HAVE_EPOXY && HAVE_EPOXY
+    epoxy_handle_external_wglMakeCurrent();
+#endif
     return true;
 }
 
@@ -550,6 +618,9 @@ void DestroyContext()
     if (context.renderingContext != NULL)
     {
         wglMakeCurrent(NULL, NULL);
+#if defined HAVE_EPOXY && HAVE_EPOXY
+        epoxy_handle_external_wglMakeCurrent();
+#endif
         wglDeleteContext(context.renderingContext);
     }
     if (context.window != NULL && context.dc != NULL)
@@ -567,9 +638,15 @@ void DestroyContext()
 static GLuint GlutWindowHandle;
 bool CreateContext(int *argcp, char **argv)
 {
+    // GLUT changes the working directory to the ressource path of the bundle
+    // so we store the old working directory and restore it at the end
+    char workingDir[FILENAME_MAX];
+    getcwd(workingDir, FILENAME_MAX);
     glutInit(argcp, argv);
     glutInitDisplayMode(GLUT_SINGLE | GLUT_RGBA | GLUT_ALPHA);
     GlutWindowHandle = glutCreateWindow("Hugin");
+    // restore working dir, as GLUT changes it the ressource path
+    chdir(workingDir);
     return true;
 }
 
@@ -578,10 +655,103 @@ void DestroyContext()
     glutDestroyWindow(GlutWindowHandle);
 }
 
+#else
+#if defined HAVE_EGL && HAVE_EGL
+#if defined HAVE_EPOXY && HAVE_EPOXY
+#include <epoxy/egl.h>
+#else
+#include <EGL/egl.h>
+#endif
+
+struct ContextSettings
+{
+    EGLDisplay m_display;
+    EGLContext m_context;
+};
+
+static ContextSettings context;
+
+static const EGLint configAttribs[] = {
+    EGL_BLUE_SIZE, 8,
+    EGL_GREEN_SIZE, 8,
+    EGL_RED_SIZE, 8,
+    EGL_ALPHA_SIZE, 8,
+    EGL_RENDERABLE_TYPE, EGL_OPENGL_BIT,
+    EGL_CONFORMANT, EGL_OPENGL_BIT, 
+    EGL_COLOR_BUFFER_TYPE, EGL_RGB_BUFFER,
+    EGL_NONE
+};
+
+bool CreateContext(int *argcp, char **argv)
+{
+    // get display connection
+    context.m_display = eglGetDisplay(EGL_DEFAULT_DISPLAY);
+    if (context.m_display == EGL_NO_DISPLAY)
+    {
+        std::cerr << "Could not connect to EGL_DEFAULT_DISPLAY" << std::endl;
+        return false;
+    };
+    // initialize egl
+    EGLint major, minor;
+    if (eglInitialize(context.m_display, &major, &minor) != EGL_TRUE)
+    {
+        std::cerr << "Could not initialize EGL" << std::endl
+            << "Error: 0x" << std::hex << eglGetError() << std::endl;
+        return false;
+    };
+    std::cout << "Init OpenGL ES " << major << "." << minor << std::endl;
+    std::cout << "Client API: " << eglQueryString(context.m_display, EGL_CLIENT_APIS) << std::endl
+        << "Vendor: " << eglQueryString(context.m_display, EGL_VENDOR) << std::endl
+        << "Version: " << eglQueryString(context.m_display, EGL_VERSION) << std::endl
+        << "EGL Extensions: " << eglQueryString(context.m_display, EGL_EXTENSIONS) << std::endl;
+    // bind OpenGL API (not OpenGL ES)
+    if (!eglBindAPI(EGL_OPENGL_API))
+    {
+        std::cerr << "Could not bind OpenGL API" << std::endl
+            << "Error: 0x" << std::hex << eglGetError() << std::endl;
+        return false;
+    };
+    // choose config
+    EGLint numConfigs;
+    EGLConfig egl_config;
+    if (eglChooseConfig(context.m_display, configAttribs, &egl_config, 1, &numConfigs) != EGL_TRUE)
+    {
+        std::cerr << "Cound not set egl config" << std::endl
+            << "Error: 0x" << std::hex << eglGetError() << std::endl;
+        return false;
+    };
+    // create surface
+    // create context and make it current
+    context.m_context = eglCreateContext(context.m_display, egl_config, EGL_NO_CONTEXT, NULL);
+    if (context.m_context == EGL_NO_CONTEXT)
+    {
+        std::cerr << "Cound not create context" << std::endl
+            << "Error: 0x" << std::hex << eglGetError() << std::endl;
+        return false;
+    };
+    if (eglMakeCurrent(context.m_display, EGL_NO_SURFACE, EGL_NO_SURFACE, context.m_context) != EGL_TRUE)
+    {
+        std::cerr << "Could not make current context" << std::endl
+            << "Error: 0x" << std::hex << eglGetError() << std::endl;
+        return false;
+    };
+    return true;
+}
+
+void DestroyContext()
+{
+    // terminate egl at end
+    eglTerminate(context.m_display);
+};
+
 #else
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
+#if defined HAVE_EPOXY && HAVE_EPOXY
+#include <epoxy/glx.h>
+#else
 #include <GL/glx.h>
+#endif
 
 struct ContextSettings
 {
@@ -670,6 +840,7 @@ void DestroyContext()
     };
 };
 #endif
+#endif
 
 bool initGPU(int *argcp, char **argv)
 {
@@ -677,6 +848,17 @@ bool initGPU(int *argcp, char **argv)
     {
         return false;
     };
+    std::cout << hugin_utils::stripPath(argv[0]) << ": using graphics card: " << glGetString(GL_VENDOR) << " " << glGetString(GL_RENDERER) << std::endl;
+#if defined HAVE_EPOXY && HAVE_EPOXY
+    const GLboolean has_arb_fragment_shader = epoxy_has_gl_extension("GL_ARB_fragment_shader");
+    const GLboolean has_arb_vertex_shader = epoxy_has_gl_extension("GL_ARB_vertex_shader");
+    const GLboolean has_arb_shader_objects = epoxy_has_gl_extension("GL_ARB_shader_objects");
+    const GLboolean has_arb_shading_language = epoxy_has_gl_extension("GL_ARB_shading_language_100");
+    const GLboolean has_ext_framebuffer = epoxy_has_gl_extension("GL_EXT_framebuffer_object");
+    const GLboolean has_arb_texture_rectangle = epoxy_has_gl_extension("GL_ARB_texture_rectangle");
+    const GLboolean has_arb_texture_border_clamp = epoxy_has_gl_extension("GL_ARB_texture_border_clamp");
+    const GLboolean has_arb_texture_float = epoxy_has_gl_extension("GL_ARB_texture_float");
+#else
     int err = glewInit();
     if (err != GLEW_OK)
     {
@@ -687,8 +869,6 @@ bool initGPU(int *argcp, char **argv)
         return false;
     }
 
-    std::cout << hugin_utils::stripPath(argv[0]) << ": using graphics card: " << glGetString(GL_VENDOR) << " " << glGetString(GL_RENDERER) << std::endl;
-
     const GLboolean has_arb_fragment_shader = glewGetExtension("GL_ARB_fragment_shader");
     const GLboolean has_arb_vertex_shader = glewGetExtension("GL_ARB_vertex_shader");
     const GLboolean has_arb_shader_objects = glewGetExtension("GL_ARB_shader_objects");
@@ -697,6 +877,7 @@ bool initGPU(int *argcp, char **argv)
     const GLboolean has_arb_texture_rectangle = glewGetExtension("GL_ARB_texture_rectangle");
     const GLboolean has_arb_texture_border_clamp = glewGetExtension("GL_ARB_texture_border_clamp");
     const GLboolean has_arb_texture_float = glewGetExtension("GL_ARB_texture_float");
+#endif
 
     if (!(has_arb_fragment_shader && has_arb_vertex_shader && has_arb_shader_objects && has_arb_shading_language && has_ext_framebuffer && has_arb_texture_rectangle && has_arb_texture_border_clamp && has_arb_texture_float)) {
         const char * msg[] = {"false", "true"};
@@ -754,4 +935,50 @@ std::string GetICCDesc(const cmsHPROFILE& profile)
     StrTrim(information);
     return information;
 }
+
+// check is icc profile is linear using cmsDetectRGBProfileGamma from LCMS2
+bool IsLinearICCProfile(const vigra::ImageImportInfo::ICCProfile& iccProfile)
+{
+#if LCMS_VERSION >= 2130
+    // function is available in lcms2 2.13.0 and later
+    if (iccProfile.empty())
+    {
+        return false;
+    }
+    cmsHPROFILE profile = cmsOpenProfileFromMem(iccProfile.data(), iccProfile.size());
+    if (profile == NULL)
+    {
+        // invalid profile
+        return false;
+    }
+    // get gamma value from profile
+    const cmsFloat64Number gamma = cmsDetectRGBProfileGamma(profile, 0.15);
+    cmsCloseProfile(profile);
+    return gamma > 0.6 && gamma < 1.4;
+#else
+    return false;
+#endif
+}
+
+/** return vector of known extensions of raw files, all lower case */
+std::vector<std::string> GetRawExtensions()
+{
+    std::vector<std::string> rawExt{ "dng", "crw", "cr2","cr3","raw","erf","raf","mrw","nef","orf","rw2","pef","srw","arw" };
+    return rawExt;
+};
+
+bool IsRawExtension(const std::string testExt)
+{
+    const std::string testExtLower = tolower(testExt);
+    const auto rawExts = GetRawExtensions();
+    for (const auto& ext : rawExts)
+    {
+        if (testExtLower.compare(ext) == 0)
+        {
+            return true;
+        };
+    };
+    return false;
+};
+
 } //namespace
